@@ -19,10 +19,8 @@ from gi.repository import Gtk, Gdk, GdkPixbuf
 import cairo
 
 from .abstract_canvas_tool import AbstractCanvasTool
-from .bottombar import DrawingAdaptativeBottomBar
 
-from .utilities import utilities_add_px_to_spinbutton
-from .utilities import utilities_show_handles_on_context
+from .utilities import utilities_add_unit_to_spinbtn
 
 class ToolCrop(AbstractCanvasTool):
 	__gtype_name__ = 'ToolCrop'
@@ -35,17 +33,22 @@ class ToolCrop(AbstractCanvasTool):
 		self.y_press = 0
 		self.move_instead_of_crop = False
 
-	def try_build_panel(self):
-		self.panel_id = 'crop'
-		self.window.options_manager.try_add_bottom_panel(self.panel_id, self)
+		builder = Gtk.Builder.new_from_resource( \
+		                  '/com/github/maoschanz/drawing/tools/ui/tool_crop.ui')
+		self.bottom_panel = builder.get_object('bottom-panel')
+		self.centered_box = builder.get_object('centered_box')
+		self.cancel_btn = builder.get_object('cancel_btn')
+		self.apply_btn = builder.get_object('apply_btn')
 
-	def build_bottom_panel(self):
-		bar = CropToolPanel(self.window)
-		self.height_btn = bar.height_btn
-		self.width_btn = bar.width_btn
+		self.height_btn = builder.get_object('height_btn')
+		self.width_btn = builder.get_object('width_btn')
+		utilities_add_unit_to_spinbtn(self.height_btn, 4, 'px')
+		utilities_add_unit_to_spinbtn(self.width_btn, 4, 'px')
 		self.width_btn.connect('value-changed', self.on_width_changed)
 		self.height_btn.connect('value-changed', self.on_height_changed)
-		return bar
+		
+		# FIXME X et Y ? top/bottom/left/right ? TODO
+		self.window.bottom_panel_box.add(self.bottom_panel)
 
 	def get_edition_status(self):
 		if self.apply_to_selection:
@@ -56,7 +59,7 @@ class ToolCrop(AbstractCanvasTool):
 ###################################################
 
 	def on_tool_selected(self, *args):
-		super().on_tool_selected()
+		self.apply_to_selection = self.selection_is_active()
 		self._x = 0
 		self._y = 0
 		if self.apply_to_selection:
@@ -111,7 +114,7 @@ class ToolCrop(AbstractCanvasTool):
 		self.cursor_name = cursor_name
 		self.window.set_cursor(True)
 
-	def on_press_on_area(self, event, surface, event_x, event_y):
+	def on_press_on_area(self, event, surface, tool_width, left_color, right_color, event_x, event_y):
 		self.x_press = event.x
 		self.y_press = event.y
 
@@ -163,22 +166,6 @@ class ToolCrop(AbstractCanvasTool):
 	def on_release_on_area(self, event, surface, event_x, event_y):
 		self.window.set_cursor(False)
 
-	############################################################################
-
-	def on_draw(self, area, cairo_context):
-		if self.apply_to_selection:
-			# print('on_draw: yes')
-			x1 = int(self._x)
-			y1 = int(self._y)
-			x2 = x1 + self.get_width()
-			y2 = y1 + self.get_height()
-			x1, x2, y1, y2 = self.get_selection().correct_coords(x1, x2, y1, y2, True)
-			utilities_show_handles_on_context(cairo_context, x1, x2, y1, y2)
-		# else:
-		# 	print('on_draw: no')
-
-	############################################################################
-
 	def crop_temp_pixbuf(self, x, y, width, height, is_selection):
 		new_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, width, height)
 		new_pixbuf.fill(0)
@@ -248,39 +235,3 @@ class ToolCrop(AbstractCanvasTool):
 	############################################################################
 ################################################################################
 
-class CropToolPanel(DrawingAdaptativeBottomBar):
-	__gtype_name__ = 'CropToolPanel'
-
-	def __init__(self, window):
-		super().__init__()
-		self.window = window
-		builder = self.build_ui('tools/ui/tool_crop.ui')
-		self.height_btn = builder.get_object('height_btn')
-		self.width_btn = builder.get_object('width_btn')
-		utilities_add_px_to_spinbutton(self.height_btn, 4, 'px')
-		utilities_add_px_to_spinbutton(self.width_btn, 4, 'px')
-		# TODO X et Y ? top/bottom/left/right ?
-		# ...
-
-	def init_adaptability(self):
-		super().init_adaptability()
-		temp_limit_size = self.centered_box.get_preferred_width()[0] + \
-		                    self.cancel_btn.get_preferred_width()[0] + \
-		                     self.apply_btn.get_preferred_width()[0]
-		self.set_limit_size(temp_limit_size)
-
-	def set_compact(self, state):
-		super().set_compact(state)
-		if state:
-			self.centered_box.set_orientation(Gtk.Orientation.VERTICAL)
-		else:
-			self.centered_box.set_orientation(Gtk.Orientation.HORIZONTAL)
-
-		# if self.scale_tool.apply_to_selection:
-		# 	self.???.set_visible(state)
-		# else:
-		# 	self.???.set_visible(state)
-
-	############################################################################
-################################################################################
-
