@@ -3,10 +3,10 @@
 from gi.repository import Gtk, Gdk
 import cairo, math
 
-from .abstract_classic_tool import AbstractClassicTool
+from .abstract_tool import AbstractAbstractTool
 from .utilities import utilities_smooth_path
 
-class ToolPencil(AbstractClassicTool):
+class ToolPencil(AbstractAbstractTool):
 	__gtype_name__ = 'ToolPencil'
 
 	def __init__(self, window, **kwargs):
@@ -14,6 +14,7 @@ class ToolPencil(AbstractClassicTool):
 		self.past_x = -1.0
 		self.past_y = -1.0
 		self._path = None
+		self.main_color = None
 		self.use_size = True
 
 		self.selected_shape_label = _("Round")
@@ -43,26 +44,42 @@ class ToolPencil(AbstractClassicTool):
 			self.selected_join_id = cairo.LineJoin.ROUND
 			self.selected_shape_label = _("Round")
 
+	def set_active_operator(self, *args):
+		state_as_string = self.get_option_value('cairo_operator')
+		if state_as_string == 'difference':
+			self.selected_operator = cairo.Operator.DIFFERENCE
+			self.selected_operator_label = _("Difference")
+		elif state_as_string == 'source':
+			self.selected_operator = cairo.Operator.SOURCE
+			self.selected_operator_label = _("Source color")
+		elif state_as_string == 'clear':
+			self.selected_operator = cairo.Operator.CLEAR
+			self.selected_operator_label = _("Eraser")
+		else:
+			self.selected_operator = cairo.Operator.OVER
+			self.selected_operator_label = _("Classic")
+
 	def get_options_label(self):
 		return _("Pencil options")
 
-	def set_options_attributes(self):
+	def get_edition_status(self):
 		self.is_smooth = self.get_option_value('pencil_smooth')
 		self.use_dashes = self.get_option_value('use_dashes')
 		self.set_active_shape()
 		self.set_active_operator()
-
-	def get_edition_status(self):
-		self.set_options_attributes()
 		label = self.label + ' - ' + self.selected_shape_label
 		if self.use_dashes:
 			label = label + ' - ' + _("With dashes")
 		return label
 
-	def on_press_on_area(self, event, surface, event_x, event_y):
+	def on_press_on_area(self, event, surface, tool_width, left_color, right_color, event_x, event_y):
 		self.x_press = event_x
 		self.y_press = event_y
-		self.set_common_values(event)
+		self.tool_width = tool_width
+		if event.button == 3:
+			self.main_color = right_color
+		else:
+			self.main_color = left_color
 		self._path = None
 
 	def on_motion_on_area(self, event, surface, event_x, event_y):
@@ -85,7 +102,6 @@ class ToolPencil(AbstractClassicTool):
 		self.past_x = -1.0
 		self.past_y = -1.0
 		operation = self.build_operation()
-		operation['is_preview'] = False
 		self.apply_operation(operation)
 
 	############################################################################
@@ -100,7 +116,6 @@ class ToolPencil(AbstractClassicTool):
 			'line_join': self.selected_join_id,
 			'use_dashes': self.use_dashes,
 			'is_smooth': self.is_smooth,
-			'is_preview': True,
 			'path': self._path
 		}
 		return operation
@@ -112,9 +127,11 @@ class ToolPencil(AbstractClassicTool):
 			return
 		self.restore_pixbuf()
 		cairo_context = cairo.Context(self.get_surface())
+		cairo_context.set_operator(operation['operator'])
 		cairo_context.set_line_cap(operation['line_cap'])
 		cairo_context.set_line_join(operation['line_join'])
 		line_width = operation['line_width']
+		cairo_context.set_line_width(line_width)
 		rgba = operation['rgba']
 		cairo_context.set_source_rgba(rgba.red, rgba.green, rgba.blue, rgba.alpha)
 		if operation['use_dashes']:
@@ -123,8 +140,7 @@ class ToolPencil(AbstractClassicTool):
 			utilities_smooth_path(cairo_context, operation['path'])
 		else:
 			cairo_context.append_path(operation['path'])
-		self.stroke_with_operator(operation['operator'], cairo_context, \
-		                                    line_width, operation['is_preview'])
+		cairo_context.stroke()
 
 	############################################################################
 ################################################################################
