@@ -2,10 +2,10 @@
 
 from gi.repository import Gtk, Gdk
 
-from .abstract_classic_tool import AbstractClassicTool
+from .abstract_tool import AbstractAbstractTool
 from .utilities import utilities_get_rgb_for_xy
 
-class ToolPicker(AbstractClassicTool):
+class ToolPicker(AbstractAbstractTool):
 	__gtype_name__ = 'ToolPicker'
 
 	def __init__(self, window, **kwargs):
@@ -20,8 +20,12 @@ class ToolPicker(AbstractClassicTool):
 			return # click outside of the surface
 		color = Gdk.RGBA(red=rgb_vals[0]/255, green=rgb_vals[1]/255, blue=rgb_vals[2]/255)
 		color.alpha = 1.0
-		if event.button == 1:
-			self.window.options_manager.set_left_color(color)
-		elif event.button == 3:
-			self.window.options_manager.set_right_color(color)
+		if event.button == 3:
+			self.window.color_popover_r.color_widget.set_rgba(color)
+		elif event.button == 1:
+			self.window.color_popover_l.color_widget.set_rgba(color)
+
 		self.window.back_to_previous()
+
+	############################################################################
+################################################################################
