@@ -19,10 +19,8 @@ from gi.repository import Gtk, Gdk, GdkPixbuf
 import cairo
 
 from .abstract_canvas_tool import AbstractCanvasTool
-from .bottombar import DrawingAdaptativeBottomBar
 
-from .utilities import utilities_add_px_to_spinbutton
-from .utilities import utilities_show_handles_on_context
+from .utilities import utilities_add_unit_to_spinbtn
 
 class ToolScale(AbstractCanvasTool):
 	__gtype_name__ = 'ToolScale'
@@ -33,20 +31,25 @@ class ToolScale(AbstractCanvasTool):
 		self.keep_proportions = False
 		self.x_press = 0
 		self.y_press = 0
-		self.add_tool_action_boolean('scale-proportions', True)
-		# self.add_tool_action_boolean('scale-deformation', False) # TODO
 
-	def try_build_panel(self):
-		self.panel_id = 'scale'
-		self.window.options_manager.try_add_bottom_panel(self.panel_id, self)
+		builder = Gtk.Builder.new_from_resource( \
+		                 '/com/github/maoschanz/drawing/tools/ui/tool_scale.ui')
+		self.bottom_panel = builder.get_object('bottom-panel')
+		self.centered_box = builder.get_object('centered_box')
+		self.cancel_btn = builder.get_object('cancel_btn')
+		self.apply_btn = builder.get_object('apply_btn')
 
-	def build_bottom_panel(self):
-		bar = ScaleToolPanel(self.window, self)
-		self.width_btn = bar.width_btn
-		self.height_btn = bar.height_btn
+		self.width_btn = builder.get_object('width_btn')
+		self.height_btn = builder.get_object('height_btn')
 		self.width_btn.connect('value-changed', self.on_width_changed)
 		self.height_btn.connect('value-changed', self.on_height_changed)
-		return bar
+		utilities_add_unit_to_spinbtn(self.height_btn, 4, 'px')
+		utilities_add_unit_to_spinbtn(self.width_btn, 4, 'px')
+
+		self.proportions_btn = builder.get_object('proportions_btn')
+		self.proportions_btn.connect('toggled', self.set_keep_proportions)
+
+		self.window.bottom_panel_box.add(self.bottom_panel)
 
 	def get_edition_status(self):
 		if self.apply_to_selection:
@@ -56,36 +59,32 @@ class ToolScale(AbstractCanvasTool):
 
 	############################################################################
 
-	def try_set_keep_proportions(self, *args):
-		# XXX this shit is a clue that this tool is not enough operation-ish
-		if self.keep_proportions == self.get_option_value('scale-proportions'):
-			return
-		self.keep_proportions = self.get_option_value('scale-proportions')
+	def set_keep_proportions(self, *args):
+		self.keep_proportions = self.proportions_btn.get_active()
 		if self.keep_proportions:
 			self.proportion = self.get_width()/self.get_height()
 
 	def on_tool_selected(self, *args):
-		super().on_tool_selected()
+		self.apply_to_selection = self.selection_is_active()
+		self.keep_proportions = False
 		if self.apply_to_selection:
-			width = self.get_selection_pixbuf().get_width()
-			height = self.get_selection_pixbuf().get_height()
+			w = self.get_selection_pixbuf().get_width()
+			h = self.get_selection_pixbuf().get_height()
 		else:
-			width = self.get_image().get_pixbuf_width()
-			height = self.get_image().get_pixbuf_height()
-		self.keep_proportions = self.get_option_value('scale-proportions')
-		self.proportion = width/height
-		self.width_btn.set_value(width)
-		self.height_btn.set_value(height)
+			w = self.get_image().get_pixbuf_width()
+			h = self.get_image().get_pixbuf_height()
+		self.width_btn.set_value(w)
+		self.height_btn.set_value(h)
+		self.keep_proportions = self.proportions_btn.get_active()
+		self.proportion = w/h
 
 	def on_width_changed(self, *args):
-		self.try_set_keep_proportions()
 		if self.keep_proportions:
 			if self.proportion != self.get_width()/self.get_height():
 				self.height_btn.set_value(self.get_width()/self.proportion)
 		self.update_temp_pixbuf()
 
 	def on_height_changed(self, *args):
-		self.try_set_keep_proportions()
 		if self.keep_proportions:
 			if self.proportion != self.get_width()/self.get_height():
 				self.width_btn.set_value(self.get_height()*self.proportion)
@@ -100,7 +99,7 @@ class ToolScale(AbstractCanvasTool):
 
 	############################################################################
 
-	def on_press_on_area(self, event, surface, event_x, event_y):
+	def on_press_on_area(self, event, surface, tool_width, left_color, right_color, event_x, event_y):
 		self.x_press = event.x
 		self.y_press = event.y
 
@@ -118,20 +117,6 @@ class ToolScale(AbstractCanvasTool):
 
 	############################################################################
 
-	def on_draw(self, area, cairo_context):
-		if self.apply_to_selection:
-			# print('on_draw: yes')
-			x1 = self.get_selection().selection_x # XXX
-			y1 = self.get_selection().selection_y # XXX
-			x2 = x1 + self.get_width()
-			y2 = y1 + self.get_height()
-			x1, x2, y1, y2 = self.get_selection().correct_coords(x1, x2, y1, y2, False)
-			utilities_show_handles_on_context(cairo_context, x1, x2, y1, y2)
-		# else:
-		# 	print('on_draw: no')
-
-	############################################################################
-
 	def build_operation(self):
 		operation = {
 			'tool_id': self.id,
@@ -157,54 +142,3 @@ class ToolScale(AbstractCanvasTool):
 
 	############################################################################
 ################################################################################
-
-class ScaleToolPanel(DrawingAdaptativeBottomBar):
-	__gtype_name__ = 'ScaleToolPanel'
-
-	def __init__(self, window, scale_tool):
-		super().__init__()
-		self.window = window
-		# knowing the tool is needed because the panel doesn't compact the same
-		# way if it's applied to the selection
-		self.scale_tool = scale_tool
-		builder = self.build_ui('tools/ui/tool_scale.ui')
-
-		self.width_btn = builder.get_object('width_btn')
-		self.height_btn = builder.get_object('height_btn')
-		utilities_add_px_to_spinbutton(self.height_btn, 4, 'px')
-		utilities_add_px_to_spinbutton(self.width_btn, 4, 'px')
-
-		self.width_label = builder.get_object('width_label')
-		self.height_label = builder.get_object('height_label')
-
-		self.separator = builder.get_object('separator')
-		self.options_btn = builder.get_object('options_btn')
-
-	def toggle_options_menu(self):
-		self.options_btn.set_active(not self.options_btn.get_active())
-
-	def init_adaptability(self):
-		super().init_adaptability()
-		temp_limit_size = self.centered_box.get_preferred_width()[0] + \
-		                    self.cancel_btn.get_preferred_width()[0] + \
-		                     self.apply_btn.get_preferred_width()[0]
-		self.set_limit_size(temp_limit_size)
-
-	def set_compact(self, state):
-		super().set_compact(state)
-		if state:
-			self.centered_box.set_orientation(Gtk.Orientation.VERTICAL)
-		else:
-			self.centered_box.set_orientation(Gtk.Orientation.HORIZONTAL)
-		self.width_label.set_visible(not state)
-		self.height_label.set_visible(not state)
-		self.separator.set_visible(not state)
-
-		# if self.scale_tool.apply_to_selection:
-		# 	self.???.set_visible(state)
-		# else:
-		# 	self.???.set_visible(state)
-
-	############################################################################
-################################################################################
-
